/********************************************************************
 FileName:      HardwareProfile - PIC32MX795F512L PIM.h
 Dependencies:  See INCLUDES section
 Processor:     PIC32 USB Microcontrollers
 Hardware:      PIC32MX795F512L PIM
 Compiler:      Microchip C32 (for PIC32)
 Company:       Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
  2.3   09/15/2008   Broke out each hardware platform into its own
                     "HardwareProfile - xxx.h" file
********************************************************************/

#ifndef HARDWARE_PROFILE_PIC32MX795F512L_PIM_H
#define HARDWARE_PROFILE_PIC32MX795F512L_PIM_H

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

	/*******************************************************************/
	/******** USB stack hardware selection options *********************/
	/*******************************************************************/
	//This section is the set of definitions required by the MCHPFSUSB
	//  framework.  These definitions tell the firmware what mode it is
	//  running in, and where it can find the results to some information
	//  that the stack needs.
	//These definitions are required by every application developed with
	//  this revision of the MCHPFSUSB framework.  Please review each
	//  option carefully and determine which options are desired/required
	//  for your application.
	
	//#define USE_SELF_POWER_SENSE_IO
//	#define tris_self_power     TRISAbits.TRISA2    // Input
	#define self_power          1
	
	//#define USE_USB_BUS_SENSE_IO
//	#define tris_usb_bus_sense  TRISBbits.TRISB5    // Input
	#define USB_BUS_SENSE       1 

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0
    
#define SYS_CLOCK 					(80000000ul)
#define GetSystemClock()            (SYS_CLOCK)
#define GetPeripheralClock()        (SYS_CLOCK/2)
#define GetInstructionClock()       (SYS_CLOCK)

#if 0
/*********************************************************************
* IOS FOR THE DISPLAY CONTROLLER
*********************************************************************/
	// Definitions for reset pin
	#define RST_TRIS_BIT    TRISAbits.TRISA7
	#define RST_LAT_BIT     LATAbits.LATA7
	
	// Definitions for RS pin
	#define RS_TRIS_BIT TRISAbits.TRISA10
	#define RS_LAT_BIT  LATAbits.LATA10
	
	// Definitions for CS pin
	#define CS_TRIS_BIT TRISAbits.TRISA9
	#define CS_LAT_BIT  LATAbits.LATA9

	// Definitions for POWER ON pin (for the BACK LIGHT)
	#define POWERON_LAT_BIT     LATDbits.LATD3
	#define POWERON_TRIS_BIT    TRISDbits.TRISD3
#endif


    // Clock values

    #define MILLISECONDS_PER_TICK       10                  // Definition for use with a tick timer
    #define TIMER_PRESCALER             TIMER_PRESCALER_8   // Definition for use with a tick timer
    #define TIMER_PERIOD                37500               // Definition for use with a tick timer

    /*******************************************************************/
    /******** CLOCK DEFINES *********************/
    /*******************************************************************/
    // Description: Macro used to enable the SD-SPI physical layer (SD-SPI.c and .h)
	#define USE_SD_INTERFACE_WITH_SPI
	
	// Define the SPI frequency
	#define SPI_FREQUENCY			(5000000)

    // Description: SD-SPI Chip Select Output bit
    #define SD_CS               LATDbits.LATD5
    // Description: SD-SPI Chip Select TRIS bit
    #define SD_CS_TRIS          TRISDbits.TRISD5
    
        
    // Description: The main SPI control register
    #define SPICON1             SPI1ACON
    // Description: The SPI status register
    #define SPISTAT             SPI1ASTAT
    // Description: The SPI Buffer
    #define SPIBUF              SPI1ABUF
    // Description: The receive buffer full bit in the SPI status register
    #define SPISTAT_RBF         SPI1ASTATbits.SPIRBF
    // Description: The bitwise define for the SPI control register (i.e. _____bits)
    #define SPICON1bits         SPI1ACONbits
    // Description: The bitwise define for the SPI status register (i.e. _____bits)
    #define SPISTATbits         SPI1ASTATbits
    // Description: The enable bit for the SPI module
    #define SPIENABLE           SPI1ACONbits.ON
    // Description: The definition for the SPI baud rate generator register (PIC32)
    #define SPIBRG			    SPI1ABRG

    // Tris pins for SCK/SDI/SDO lines

    // Description: The TRIS bit for the SCK pin
    #define SPICLOCK            TRISDbits.TRISD1
    // Description: The TRIS bit for the SDI pin
    #define SPIIN               TRISDbits.TRISD2
    // Description: The TRIS bit for the SDO pin
    #define SPIOUT              TRISDbits.TRISD3
    
   	//SPI Configuration
	#define SPI_START_CFG_1     (SPI_OPEN_MSTEN | SPI_OPEN_MODE8 | SPI_OPEN_CKE_REV | SPI_OPEN_ON)
    #define SPI_START_CFG_2     (GetPeripheralClock() / SPI_FREQUENCY)
    
    //SPI library functions
    #define putcSPI(data)       SpiChnPutC(SPI_CHANNEL1A, data)
	#define getcSPI()           SpiChnGetC(SPI_CHANNEL1A)
	#define OpenSPI(config1, config2)		SpiChnOpen(SPI_CHANNEL1A, config1, config2)
    
    // Will generate an error if the clock speed is too low to interface to the card
    #if (GetSystemClock() < 100000)
        #error Clock speed must exceed 100 kHz
    #endif    

    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     TRISAbits.TRISA2    // Input
    #define self_power          1

    //#define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISBbits.TRISB5    // Input
    #define USB_BUS_SENSE       1 

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
//    #define DEMO_BOARD PIC32MX460F512L_PIM
//    #define EXPLORER_16

    /** LED ************************************************************/
    #define mInitAllLEDs()      TRISBbits.TRISB15=0;TRISCbits.TRISC1=0;
    
    #define mLED_1              LATBbits.LATB15
    #define mLED_2              LATCbits.LATC1
    #define mLED_3              
    #define mLED_4              
    
    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mGetLED_3()         1
    #define mGetLED_4()         1

    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_3_On()         
    #define mLED_4_On()         
    
    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_3_Off()        
    #define mLED_4_Off()        
    
    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;
    #define mLED_3_Toggle()     
    #define mLED_4_Toggle()   
    
    /** SWITCH *********************************************************/
    #define mInitSwitch2()      TRISEbits.TRISE8=1;
    #define mInitSwitch3()      
    #define mInitAllSwitches()  mInitSwitch2();mInitSwitch3();
    #define sw2                 (PORTEbits.RE8)
    #define sw3                 


    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0





/*********************************************************************
* RTCC DEFAULT INITIALIZATION (these are values to initialize the RTCC
*********************************************************************/
    #define RTCC_DEFAULT_DAY        18      // 18
    #define RTCC_DEFAULT_MONTH      11      // November
    #define RTCC_DEFAULT_YEAR       9       // 2009
    #define RTCC_DEFAULT_WEEKDAY    05      // Friday
    #define RTCC_DEFAULT_HOUR       10      // 10:10:01
    #define RTCC_DEFAULT_MINUTE     10
    #define RTCC_DEFAULT_SECOND     01


/* External SMSC PHY configuration */
	#define	PHY_RMII				// external PHY runs in RMII mode
	// #define	PHY_CONFIG_ALTERNATE	// alternate configuration used	// SPP
	#define	PHY_ADDRESS			0x1	// the address of the National DP83848 PHY

#endif  //HARDWARE_PROFILE_PIC32MX795F512L_PIM_H
