/*************************************************************************
 *
 *
 *    (c) Copyright Olimex 2010
 *
 *    File name   : main.c
 *    Description : contains main() function
 *
 *    History :
 *    1. Date        : 02 Jun 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/

/** INCLUDES *******************************************************/
#include "includes.h"

/** CONFIGURATION **************************************************/
#pragma config UPLLEN   = ON        	// USB PLL Enabled
#pragma config FPLLMUL  = MUL_20        // PLL Multiplier
#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1        // Peripheral Clock divisor
#pragma config FWDTEN   = OFF           // Watchdog Timer
#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
//#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = ON           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1      // ICE/ICD Comm Channel Select		// SPP
#pragma config DEBUG    = ON            // Background Debugger Enable
#pragma config FVBUSONIO = OFF			// VBUS_ON pin is controlled by the Port Function
#pragma config FUSBIDIO = OFF			// USBID pin is controlled by the Port Function 
#pragma config FSRSSEL = PRIORITY_7		//SRS Interrupt Priority Level 7
#pragma config FMIIEN = OFF, FETHIO = ON	// external PHY in RMII/default configuration	// SPP
#pragma config FCANIO = OFF


/** VARIABLES ******************************************************/
#pragma udata
#define USB_IO_BUF_SIZE 256
char USB_In_Buffer[USB_IO_BUF_SIZE];
char USB_Out_Buffer[USB_IO_BUF_SIZE];


/********************************************************************
 * Function:        int main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *******************************************************************/
int main(void)
{
	char status;


    InitializeSystem();
	// create tasks
	status = 0;
	status |= TaskControlsInit();
	status |= vTaskTCPIPInitialize();
	status |= vTaskIOInitialize();
	//status |= vTaskUSBMouseInitialize();
	if(status)
		while(1);
	
	/* Finally start the scheduler. */
	vTaskStartScheduler();

	/* Will only reach here if there is insufficient heap available to start
	the scheduler. */
	Nop();
	Nop();
	Nop();
	while(1);

// should never get here!!!
	testing();

    MouseUsbTask();
}

/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void)
{
	// Default all pins to digital
	AD1PCFG = 0xFFFF;
	
	// disable JTAG support
	mJTAGPortEnable(0);
	
	UserPeriphInit();
	
	// enable interrupts
	SYSTEMConfigPerformance( GetSystemClock() -1 );
	INTEnableSystemMultiVectoredInt();
	
	mOSCSetPBDIV(OSC_PB_DIV_2);
	
	portDISABLE_INTERRUPTS();
	
	//Initialize the RTCC
	RtccInit();
	while(RtccGetClkStat()!=RTCC_CLK_ON);// wait for the SOSC to be actually running and RTCC to have its clock source
	// could wait here at most 32ms
	RtccOpen(0x10073000, 0x07011602, 0);
	UserInit ();
}

/******************************************************************************
* Function:        void UserInit(void)
*
* PreCondition:    None
*
* Input:           None
*
* Output:          None
*
* Side Effects:    None
*
* Overview:        This routine should take care of all of the demo code
*                  initialization that is required.
*
* Note:            
*
*****************************************************************************/
void UserInit(void)
{
	//TRISB = TRISB & (~(0x8000));
	// enable power supply to peripherals
	TRISBbits.TRISB13 = 0;
	ODCBbits.ODCB13 = 1;
	LATBbits.LATB13 = 0;
	
	//Initialize all of the LED pins
	mInitAllLEDs();
	mLED_1_Off();
	mLED_2_Off();

	//Initialize the pushbuttons
	mInitAllSwitches();
	
	// init analogue inputs
	PORTSetPinsAnalogIn(IOPORT_B, BIT_2);	
	
	USB_Out_Buffer[0] = 0;
}//end UserInit

/********************************************************************
* Function:        void ProcessIO(void)
*
* PreCondition:    None
*
* Input:           None
*
* Output:          None
*
* Side Effects:    None
*
* Overview:        This function is a place holder for other user
*                  routines. It is a mixture of both USB and
*                  non-USB tasks.
*
* Note:            None
*******************************************************************/
void ProcessIO(void)
{
/*
	volatile long int x;
	
	LATB = LATB ^ 0x8000;
	for (x=1000000; x; x--);
*/	
	#if 0
	// When a test is started LED2 goes on
	// When the test is complete and successfull LED2 goes off
	// When the test is complete and an error occures LED2 starts blinking
	// When all tests are over LED1 starts blinking
	
	// A test is started by pressing SPACE
	// If there is an error durnig the test the you can restart it by pressing SPACE again
	// If you wish to skip testing press 'N'
	
	static enum {
		TEST_NONE,
		TEST_ONBOARD_PINS,
		TEST_EXTENTION_ARDUINO,
		TEST_EXTENTION_GPIO,
		TEST_EXTENTION_UEXT,
		TEST_EXTENTION_UEXT_OPT,
		TEST_RTC,
		TEST_SDCARD,
		TEST_CAN,
		TEST_USBOTG,
		TEST_END
	} testToRun = TEST_ONBOARD_PINS;
	
	static BOOL testJustStarted = FALSE;
	static BOOL testRunning = FALSE;
	static BOOL errorDetected = FALSE;
	static BOOL allTestsFinished = FALSE;
	
	int result = 0;
	int i;

	// process user input
	switch(USB_Out_Buffer[0]) {
	case '\0': // get here if nothing has been received
		break;
		
	case 'n':
	case 'N':
		if(errorDetected)
			++testToRun;
		else
			break;
	case ' ':
		if(!testJustStarted) {
			result = 0;
			errorDetected = FALSE;
			testJustStarted = TRUE;	
			testRunning = TRUE;
			mLED_2_On();
			mLED_1_Off();
			Delayms(10);
		
		
		
		
		} else {
			testJustStarted = FALSE;
			testRunning = TRUE;
		}	
		break;

	default: // get here if input is incorrect
		break;	
	}	
	
	if(testRunning) {
		switch(testToRun) {
		case TEST_NONE:
			break;
		case TEST_ONBOARD_PINS:
			result = TestOnboardPins(testJustStarted, USB_In_Buffer);
			break;
		case TEST_EXTENTION_ARDUINO:
			result = ExtConTest(&ARDUINOEXT_desc, testJustStarted, USB_In_Buffer);
			Delayms(50);
			if(testJustStarted)
				sprintf(USB_In_Buffer, "\r\nStarted test of ARDUINOEXT:\r\n");
			break;
			
		case TEST_EXTENTION_GPIO:
			result = ExtConTest(&GPIOEXT_desc, testJustStarted, USB_In_Buffer);
			Delayms(50);
			if(testJustStarted)
				sprintf(USB_In_Buffer, "\r\nStarted test of GPIOEXT:\r\n");
			break;			
						
		case TEST_EXTENTION_UEXT:
			result = ExtConTest(&UEXT_desc, testJustStarted, USB_In_Buffer);
			Delayms(50);
			if(testJustStarted)
				sprintf(USB_In_Buffer, "\r\nStarted test of UEXT:\r\n");
			break;
		case TEST_EXTENTION_UEXT_OPT:
			result = ExtConTest(&UEXT_desc, testJustStarted, USB_In_Buffer);
			Delayms(50);
			if(testJustStarted)
				sprintf(USB_In_Buffer, "\r\nStarted test of UEXT_OPT:\r\n");
			break;
			
		case TEST_RTC:			
			result = rtc_test(testJustStarted, USB_In_Buffer);
			break;
			
		case TEST_SDCARD:
			result = TestSdCard(testJustStarted, USB_In_Buffer);
			break;
			
		case TEST_USBOTG:
			result = TestUsbOtg(testJustStarted, USB_In_Buffer);
			break;		
			
		case TEST_CAN:
			result = TestCAN_GPIO(testJustStarted, USB_In_Buffer);
			break;		
			
		case TEST_END:
			allTestsFinished = TRUE;
			break;
					
		default:
			result = -1;
			break;
		}
		
		if(testJustStarted)
			testRunning = FALSE;
	}
	
	// indicate test result on LED2
	// LED2 goes off when test is over and has run successfully
	if(result) {
		testRunning = FALSE;

		mInitSwitch2();
		mInitAllLEDs();
		if(result == -1) {			
			errorDetected = TRUE;
			strcat(USB_In_Buffer, "\r\nTest FAILED!!\r\nPress SPACE to run again or 'N' to skip");
		} else if(result == 1) {
			mLED_2_Off();

			strcat(USB_In_Buffer, "\r\nTest ran SUCCESSFULLY");
			
			// move to the next test
			if(++testToRun == TEST_END)
				allTestsFinished = TRUE;
		}
			
		result = 0;
	}	
	
	if(errorDetected) {
		// in case of error blink LED fast
		Delayms(100);
		mLED_2_Toggle();
	} else if(allTestsFinished) {
		// if all tests are over blink LED slowly
		Delayms(500);
		mLED_1_Toggle();


		// to complete the test we should get into sleep mode only if device is
		// battery powered. This can be detected by the state of the RG7 pin. If
		// the pin is HI, then +5V is present and we can conclude that device is
		// not currently powered by the battery.
		TRISGbits.TRISG7 = 1;

		if(!sw2) {
			mLED_2_On();

			if(!PORTGbits.RG7) {
				mLED_2_Toggle();
				Delayms(500);
				mLED_2_Toggle();
				Delayms(500);
				mLED_2_Toggle();
				Delayms(500);

				mLED_2_Off();
				mLED_1_Off();

				// disable uart
				UARTEnable(UART3B, UART_DISABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));

				// disable USB
				U1PWRCbits.USBPWR = 0;

				TRISB	=	0xA906;
				LATB	=	0x0000;

				TRISC	=	0xFFFF;
				LATC	=	0x0000;

				TRISD	=	0x07FD;
				LATD	=	0x0000;

				TRISE	=	0x0003;
				LATE	=	0x0000;

				TRISF	=	0x0001;
				LATF	=	0x0000;

				TRISG	=	0x0080;
				LATG	=	0x0000;

				// turn off peripherals
				LATDbits.LATD11 = 1;

				// BAT and ID pins should be analogue
				PORTSetPinsAnalogIn(IOPORT_B, BIT_1 | BIT_2 | BIT_15);

				// enter sleep now
				INTDisableInterrupts();
				PowerSaveSleep();
			}
		} else {
			mLED_2_Off();
		}
	}
	
	// communicate over RS232
	UartCommDoWork();
	
	/* WRITE TO RS232 */
	if(USB_In_Buffer[0]) {
		UartCommWrite(USB_In_Buffer, strlen(USB_In_Buffer));
		USB_In_Buffer[0] = 0;
	}
	
	/* READ FROM RS232 */
	i = UartCommRead(USB_Out_Buffer, USB_IO_BUF_SIZE);
	USB_Out_Buffer[i] = 0; // terminate string	
	
	UartCommDoWork();
	#endif
}



void vApplicationIdleHook( void )
{
	/* Schedule the co-routines from within the idle task hook. */
//	vCoRoutineSchedule();
}

void vApplicationTickHook( void )
{

}

/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( void )
{
	/* Look at pxCurrentTCB to see which task overflowed its stack. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void _general_exception_handler( unsigned portLONG ulCause, unsigned portLONG ulStatus )
{
	/* This overrides the definition provided by the kernel.  Other exceptions 
	should be handled here. */
	for( ;; );
}
/*-----------------------------------------------------------*/
void vApplicationMallocFailedHook( void )
{
	for( ;; );	
}
